/* vi:set ts=8 sts=0 sw=8:
 * $Id: prefs.h,v 1.26 1999/03/04 18:43:13 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __PREFS_H__
#define __PREFS_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#include <gtk/gtk.h>
#include "main.h"

/* general/global options and macros */
#define AUTO_INDENT			(1 << 0)
#define SHOW_TABS			(1 << 1)
#define SHOW_MSGBAR			(1 << 2)
#define SHOW_TOOLBAR			(1 << 3)
#define PIC_TEXT_TOOLBAR		(1 << 4)
#define TEXT_ONLY_TOOLBAR		(1 << 5)
#define PIC_ONLY_TOOLBAR		(1 << 6)
#define SHOW_TOOLTIPS			(1 << 7)
#define USE_WORDWRAP			(1 << 8)
#define SAVE_WIN_WIDTH			(1 << 9)
#define SAVE_WIN_HEIGHT			(1 << 10)
#define SAVE_WIN_POS			(1 << 11)
#define USE_MSGBOX			(1 << 12)
#define SHOW_PRJBAR			(1 << 13)
#define TOOLBAR_RAISED			(1 << 14)
#define SHOW_HTML_TOOLBAR		(1 << 15)
#define SHOW_SPLASH			(1 << 16)
#define DO_FORK				(1 << 17)
#define DO_BACKUP			(1 << 18)
#define DOC_TABS_TOP			(1 << 19)
#define DOC_TABS_BOTTOM			(1 << 20)
#define DOC_TABS_LEFT			(1 << 21)
#define DOC_TABS_RIGHT			(1 << 22)

#define DEFAULT_OPTIONS 		(AUTO_INDENT      | SHOW_TABS     | \
					 SHOW_MSGBAR      | SHOW_TOOLBAR  | \
					 PIC_ONLY_TOOLBAR | SHOW_TOOLTIPS | \
					 SAVE_WIN_HEIGHT  | USE_MSGBOX | \
					 SHOW_SPLASH      | DO_FORK | \
					 DO_BACKUP        | DOC_TABS_TOP | \
					 SHOW_HTML_TOOLBAR)

/* these macros operate on the global variable 'options' */
#define IS_AUTO_INDENT()		(prefs.options1 & AUTO_INDENT)
#define IS_SHOW_TABS()			(prefs.options1 & SHOW_TABS)
#define IS_SHOW_MSGBAR()		(prefs.options1 & SHOW_MSGBAR)
#define IS_SHOW_TOOLBAR()		(prefs.options1 & SHOW_TOOLBAR)
#define IS_TEXT_PIC_TOOLBAR()		(prefs.options1 & PIC_TEXT_TOOLBAR)
#define IS_TEXT_TOOLBAR()		(prefs.options1 & TEXT_ONLY_TOOLBAR)
#define IS_PIC_TOOLBAR()		(prefs.options1 & PIC_ONLY_TOOLBAR)
#define IS_SHOW_TOOLTIPS()		(prefs.options1 & SHOW_TOOLTIPS)
#define IS_USE_WORDWRAP()		(prefs.options1 & USE_WORDWRAP)
#define IS_USE_MSGBOX()			(prefs.options1 & USE_MSGBOX)
#define IS_SAVE_WIN_WIDTH()		(prefs.options1 & SAVE_WIN_WIDTH)
#define IS_SAVE_WIN_HEIGHT()		(prefs.options1 & SAVE_WIN_HEIGHT)
#define IS_SAVE_WIN_POS()		(prefs.options1 & SAVE_WIN_POS)
#define IS_SHOW_PRJBAR()		(prefs.options1 & SHOW_PRJBAR)
#define IS_TOOLBAR_RAISED()		(prefs.options1 & TOOLBAR_RAISED)
#define IS_SHOW_HTML_TOOLBAR()		(prefs.options1 & SHOW_HTML_TOOLBAR)
#define IS_SHOW_SPLASH()		(prefs.options1 & SHOW_SPLASH)
#define IS_DO_FORK()			(prefs.options1 & DO_FORK)
#define IS_DO_BACKUP()			(prefs.options1 & DO_BACKUP)
#define IS_DOC_TABS_TOP()		(prefs.options1 & DOC_TABS_TOP)
#define IS_DOC_TABS_BOTTOM()		(prefs.options1 & DOC_TABS_BOTTOM)
#define IS_DOC_TABS_LEFT()		(prefs.options1 & DOC_TABS_LEFT)
#define IS_DOC_TABS_RIGHT()		(prefs.options1 & DOC_TABS_RIGHT)

#define SET_SHOW_TABS()			(prefs.options1 |= SHOW_TABS)
#define SET_SHOW_MSGBAR()		(prefs.options1 |= SHOW_MSGBAR)
#define SET_SHOW_TOOLBAR()		(prefs.options1 |= SHOW_TOOLBAR)
#define SET_TEXT_ONLY_TOOLBAR()		(prefs.options1 |= TEXT_ONLY_TOOLBAR)
#define SET_PIC_ONLY_TOOLBAR()		(prefs.options1 |= PIC_ONLY_TOOLBAR)
#define SET_PIC_TEXT_TOOLBAR()		(prefs.options1 |= PIC_TEXT_TOOLBAR)
#define SET_SHOW_TOOLTIPS()		(prefs.options1 |= SHOW_TOOLTIPS)
#define SET_USE_WORDWRAP()		(prefs.options1 |= USE_WORDWRAP)
#define SET_SHOW_PRJBAR()		(prefs.options1 |= SHOW_PRJBAR)
#define SET_TOOLBAR_RAISED()		(prefs.options1 |= TOOLBAR_RAISED)
#define SET_SHOW_HTML_TOOLBAR()		(prefs.options1 |= SHOW_HTML_TOOLBAR)
#define SET_DOC_TABS_TOP()		(prefs.options1 |= DOC_TABS_TOP)
#define SET_DOC_TABS_BOTTOM()		(prefs.options1 |= DOC_TABS_BOTTOM)
#define SET_DOC_TABS_LEFT()		(prefs.options1 |= DOC_TABS_LEFT)
#define SET_DOC_TABS_RIGHT()		(prefs.options1 |= DOC_TABS_RIGHT)

#define CLEAR_SHOW_TABS()		(prefs.options1 &= ~SHOW_TABS)
#define CLEAR_SHOW_MSGBAR()		(prefs.options1 &= ~SHOW_MSGBAR)
#define CLEAR_SHOW_TOOLBAR()		(prefs.options1 &= ~SHOW_TOOLBAR)
#define CLEAR_PIC_TEXT_TOOLBAR()	(prefs.options1 &= ~PIC_TEXT_TOOLBAR)
#define CLEAR_TEXT_ONLY_TOOLBAR()	(prefs.options1 &= ~TEXT_ONLY_TOOLBAR)
#define CLEAR_PIC_ONLY_TOOLBAR()	(prefs.options1 &= ~PIC_ONLY_TOOLBAR)
#define CLEAR_SHOW_TOOLTIPS()		(prefs.options1 &= ~SHOW_TOOLTIPS)
#define CLEAR_USE_WORDWRAP()		(prefs.options1 &= ~USE_WORDWRAP)
#define CLEAR_USE_MSGBOX()		(prefs.options1 &= ~USE_MSGBOX)
#define CLEAR_SHOW_PRJBAR()		(prefs.options1 &= ~SHOW_PRJBAR)
#define CLEAR_TOOLBAR_RAISED()		(prefs.options1 &= ~TOOLBAR_RAISED)
#define CLEAR_SHOW_HTML_TOOLBAR()	(prefs.options1 &= ~SHOW_HTML_TOOLBAR)
#define CLEAR_DOC_TABS_TOP()		(prefs.options1 &= ~DOC_TABS_TOP)
#define CLEAR_DOC_TABS_BOTTOM()		(prefs.options1 &= ~DOC_TABS_BOTTOM)
#define CLEAR_DOC_TABS_LEFT()		(prefs.options1 &= ~DOC_TABS_LEFT)
#define CLEAR_DOC_TABS_RIGHT()		(prefs.options1 &= ~DOC_TABS_RIGHT)

/* main toolbar options */
#define SHOW_MAIN_NEW			(1 << 0)
#define SHOW_MAIN_OPEN			(1 << 1)
#define SHOW_MAIN_CLOSE			(1 << 2)
#define SHOW_MAIN_SAVE			(1 << 3)
#define SHOW_MAIN_PRINT			(1 << 4)
#define SHOW_MAIN_CUT			(1 << 5)
#define SHOW_MAIN_COPY			(1 << 6)
#define SHOW_MAIN_PASTE			(1 << 7)
#define SHOW_MAIN_PREFS			(1 << 8)
#define SHOW_MAIN_EXIT			(1 << 9)
#define SHOW_MAIN_FIND			(1 << 10)
#define SHOW_MAIN_FIND_NEXT		(1 << 11)
#define SHOW_MAIN_REPLACE		(1 << 12)
#define SHOW_MAIN_UNDO			(1 << 13)
#define SHOW_MAIN_REDO			(1 << 14)
#define SHOW_MAIN_WINNEW		(1 << 15)
#define SHOW_MAIN_WINCLOSE		(1 << 16)
 
#define DEFAULT_MAIN_TB_OPT		(SHOW_MAIN_NEW   | SHOW_MAIN_OPEN  | \
					 SHOW_MAIN_CLOSE | SHOW_MAIN_SAVE  | \
					 SHOW_MAIN_PRINT | SHOW_MAIN_CUT   | \
					 SHOW_MAIN_COPY  | SHOW_MAIN_PASTE | \
					 SHOW_MAIN_PREFS | SHOW_MAIN_EXIT  | \
					 SHOW_MAIN_FIND  | SHOW_MAIN_REPLACE | \
					 SHOW_MAIN_UNDO  | SHOW_MAIN_REDO | \
					 SHOW_MAIN_WINNEW| \
					 SHOW_MAIN_WINCLOSE | \
					 SHOW_MAIN_FIND_NEXT)

#define MAIN_TB_STR_NEW			"main_tb_new"
#define MAIN_TB_STR_OPEN		"main_tb_open"
#define MAIN_TB_STR_CLOSE		"main_tb_close"
#define MAIN_TB_STR_SAVE		"main_tb_save"
#define MAIN_TB_STR_PRINT		"main_tb_print"
#define MAIN_TB_STR_CUT			"main_tb_cut"
#define MAIN_TB_STR_COPY		"main_tb_copy"
#define MAIN_TB_STR_UNDO		"main_tb_undo"
#define MAIN_TB_STR_REDO		"main_tb_redo"
#define MAIN_TB_STR_PASTE		"main_tb_paste"
#define MAIN_TB_STR_FIND		"main_tb_find"
#define MAIN_TB_STR_FNEXT		"main_tb_fnext"
#define MAIN_TB_STR_REPLACE		"main_tb_replace"
#define MAIN_TB_STR_PREFS		"main_tb_prefs"
#define MAIN_TB_STR_EXIT		"main_tb_exit"
#define MAIN_TB_STR_WINNEW		"main_tb_winnew"
#define MAIN_TB_STR_WINCLOSE		"main_tb_winclose"


/* html toolbar options */
#define SHOW_HTML_TTL			(1 << 0)
#define SHOW_HTML_LINK			(1 << 1)
#define SHOW_HTML_TARG			(1 << 2)
#define SHOW_HTML_IMG			(1 << 3)
#define SHOW_HTML_CMNT			(1 << 4)
#define SHOW_HTML_H1			(1 << 5)
#define SHOW_HTML_H2			(1 << 6)
#define SHOW_HTML_H3			(1 << 7)
#define SHOW_HTML_H4			(1 << 8)
#define SHOW_HTML_H5			(1 << 9)
#define SHOW_HTML_H6			(1 << 10)
#define SHOW_HTML_LEFT			(1 << 11)
#define SHOW_HTML_CENT			(1 << 12)
#define SHOW_HTML_RGHT			(1 << 13)
#define SHOW_HTML_BOLD			(1 << 14)
#define SHOW_HTML_ITAL			(1 << 15)
#define SHOW_HTML_ULNE			(1 << 16)
#define SHOW_HTML_TTY			(1 << 17)
#define SHOW_HTML_STRK			(1 << 18)
#define SHOW_HTML_EMPH			(1 << 19)
#define SHOW_HTML_STRG			(1 << 20)
#define SHOW_HTML_SUB			(1 << 21)
#define SHOW_HTML_SUP			(1 << 22)
#define SHOW_HTML_SEP			(1 << 23)
#define SHOW_HTML_PARA			(1 << 24)
#define SHOW_HTML_LBRK			(1 << 25)
#define SHOW_HTML_BIG			(1 << 26)
#define SHOW_HTML_SMLL			(1 << 27)
#define SHOW_HTML_ORDR			(1 << 28)
#define SHOW_HTML_UORD			(1 << 29)
#define SHOW_HTML_LITM			(1 << 30)

#define DEFAULT_HTML_TB_OPT		(SHOW_HTML_TTL  | SHOW_HTML_LINK | \
					 SHOW_HTML_TARG | SHOW_HTML_IMG  | \
					 SHOW_HTML_CMNT | SHOW_HTML_H1   | \
					 SHOW_HTML_H2   | SHOW_HTML_H3   | \
					 SHOW_HTML_LEFT | SHOW_HTML_CENT | \
					 SHOW_HTML_RGHT | SHOW_HTML_BOLD | \
					 SHOW_HTML_ITAL | SHOW_HTML_ULNE | \
					 SHOW_HTML_TTY  | SHOW_HTML_STRK | \
					 SHOW_HTML_EMPH | SHOW_HTML_STRG | \
					 SHOW_HTML_SUB  | SHOW_HTML_SUP  | \
					 SHOW_HTML_SEP  | SHOW_HTML_PARA | \
					 SHOW_HTML_LBRK | SHOW_HTML_BIG  | \
					 SHOW_HTML_SMLL | SHOW_HTML_ORDR | \
					 SHOW_HTML_UORD | SHOW_HTML_LITM)

#define HTML_TB_STR_BIG			"html_tb_big"
#define HTML_TB_STR_BOLD		"html_tb_bold"
#define HTML_TB_STR_CENT		"html_tb_cent"
#define HTML_TB_STR_CMNT		"html_tb_cmnt"
#define HTML_TB_STR_EMPH		"html_tb_emph"
#define HTML_TB_STR_H1			"html_tb_h1"
#define HTML_TB_STR_H2			"html_tb_h2"
#define HTML_TB_STR_H3			"html_tb_h3"
#define HTML_TB_STR_H4			"html_tb_h4"
#define HTML_TB_STR_H5			"html_tb_h5"
#define HTML_TB_STR_H6			"html_tb_h6"
#define HTML_TB_STR_IMG			"html_tb_img"
#define HTML_TB_STR_ITAL		"html_tb_ital"
#define HTML_TB_STR_LBRK		"html_tb_lbrk"
#define HTML_TB_STR_LEFT		"html_tb_left"
#define HTML_TB_STR_LINK		"html_tb_link"
#define HTML_TB_STR_LITM		"html_tb_litm"
#define HTML_TB_STR_ORDR		"html_tb_ordr"
#define HTML_TB_STR_PARA		"html_tb_para"
#define HTML_TB_STR_RGHT		"html_tb_rght"
#define HTML_TB_STR_SEP			"html_tb_sep"
#define HTML_TB_STR_SMLL		"html_tb_smll"
#define HTML_TB_STR_STRG		"html_tb_strg"
#define HTML_TB_STR_STRK		"html_tb_strk"
#define HTML_TB_STR_SUB			"html_tb_sub"
#define HTML_TB_STR_SUP			"html_tb_sup"
#define HTML_TB_STR_TARG		"html_tb_targ"
#define HTML_TB_STR_TTL			"html_tb_ttl"
#define HTML_TB_STR_TTY			"html_tb_tty"
#define HTML_TB_STR_ULNE		"html_tb_ulne"
#define HTML_TB_STR_UORD		"html_tb_uord"

typedef struct {

	/* bit flag options for various categories */
	long		options1;	/* see above for descriptions */
	long		main_tb_opt;	/* options for main toolbar */
	long		html_tb_opt;	/* options for html toolbar */

	/* non user configurable */
	char *		appdir;		/* $HOME/PREFS_DIR/ */

	/* appearance settings (most are bit-flag settings) */
	GtkPositionType	tabpos;		/* doc tab position */

	/* document settings */
#ifdef APP_GNP
	guint		autosave;	/* autosave minutes (0 = disables) */
	guint		random_tips;	/* random tips sec. (0 = disables) */
#endif
	ubyte		maxrecent;	/* max num of recently opened docs */
	char *		backupdir;	/* backup directory (default = ./) */
	char *		backupsuffix;	/* backup file's suffix (def = ~) */

	/* window settings */
	int		win_height;	/* window height */
	int		win_width;	/* window width */
	int		win_xpos;	/* window xposition */
	int		win_ypos;	/* window yposition */

#ifdef APP_GNP
	/* fonts and colors */
	GdkColor	text_fg_color;	/* GDK-styled text fg color */
	char *		text_fg_str;	/* fg color (rgb string) */
	GdkColor	text_bg_color;	/* GDK-styled text bg color */
	char *		text_bg_str;	/* bg color (rgb string) */
	GdkFont *	text_font;	/* GDK-styled text font */
	char *		text_font_str;	/* text font string */
	GdkColor	text_hlfg_color;/* GDK-styled text highlight fg color */
	char *		text_hlfg_str;	/* highlight fg color (rgb string) */
	GdkColor	text_hlbg_color;/* GDK-styled text highlight bg color */
	char *		text_hlbg_str;	/* highlight bg color (rgb string) */
#endif

	/* misc */
	char *		printcmd;	/* print cmd. 'lpr' */
	char *		tmpdir;		/* temp directory. '/var/tmp' */
	guint		msgbox_max_msg;	/* max num of msgs in msg box */
	guint		msgbox_per_del;	/* % of msgs to delete in msg box */
	char *		shell;		/* user shell */

} app_prefs_t;

extern app_prefs_t prefs;


extern void	prefs_init_pre(void);
extern void	prefs_init_post(void);
extern void	prefs_save(void);
extern void	prefs_cb(GtkWidget *wgt, gpointer cbdata);
extern bool_t	prefs_bool_by_name(char *prefname);
#ifdef APP_GNP
extern void	prefs_text_color_update(void);
extern void	prefs_update_text_widget_style(GtkWidget *wgt);
#endif	/* APP_GNP */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	/* __PREFS_H__ */
